/* $Id: iprofd.c,v 1.1 1995/12/18 18:22:24 fritz Exp fritz $
 *
 * Daemon for saving ttyIx-profiles to a file.
 *
 * Copyright 1994,95 by Fritz Elfert (fritz@wuemaus.franken.de)
 * Copyright 1995 Thinking Objects Software GmbH Wuerzburg
 *
 * This file is part of Isdn4Linux.
 * 
 * Isdn4Linux is distributed with NO WARRANTY OF ANY KIND.  No author
 * or distributor accepts any responsibility for the consequences of using it,
 * or for whether it serves any particular purpose or works at all, unless he
 * or she says so in writing.  Refer to the Isdn4Linux Free Public
 * License (the "License") for full details.
 * 
 * Every copy of Isdn4Linux must include a copy of the License,
 * normally in a plain ASCII text file named LICENSE.  The License grants you
 * the right to copy, modify and redistribute Isdn4Linux, but only
 * under certain conditions described in the License.  Among other things, the
 * License requires that the copyright notice and this notice be preserved on
 * all copies.
 *
 * $Log: iprofd.c,v $
 * Revision 1.1  1995/12/18  18:22:24  fritz
 * Initial revision
 *
 */

#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/mman.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <signal.h>

#include <isdn.h>

typedef unsigned char uchar;

int  isdnctrl_fd;
char *modemsettings;

void
dumpModem(int dummy) {
  int fd;
  int len;
  char buffer[4096];

  if ((len = ioctl(isdnctrl_fd,ISDN_IOCTL_PROFILEGET,&buffer))<0) {
    perror("ioctl ISDN_IOCTL_PROFILEGET");
    exit(-1);
  }
  fd = open(modemsettings,O_WRONLY|O_CREAT|O_TRUNC,0644);
  if (fd<0) {
    perror(modemsettings);
    exit(-1);
  }
  write(fd,buffer,len);
  close(fd);
  signal(SIGIO,dumpModem);
}

void
readModem(void) {
  int len;
  int fd;
  char buffer[4096];

  fd = open(modemsettings,O_RDONLY);
  if (fd<0)
    return;
  len = read(fd,buffer,4096);
  if (len<0) {
    perror(modemsettings);
    exit(-1);
  }
  close(fd);
  if (ioctl(isdnctrl_fd,ISDN_IOCTL_PROFILESET,&buffer)<0) {
    perror("ioctl ISDN_IOCTL_PROFILESET");
    exit(-1);
  }
}

void
usage(void) {
  fprintf(stderr,"usage: iprofd <IsdnModemProfile>\n");
  exit(-1);
}

void
main(int argc, char **argv) {

  if (argc != 2)
    usage();
  modemsettings = argv[1];
  isdnctrl_fd = open("/dev/isdnctrl",O_RDONLY);
  readModem();
  switch (fork()) {
    case -1:
      perror("fork");
      exit(-1);
      break;
    case 0:      
      dumpModem(0);
      if (ioctl(isdnctrl_fd,ISDN_IOCTL_PROFILESIG,0)) {
	perror("ioctl ISDN_IOCTL_PROFILESIG");
	exit(-1);
      }
      while (1) sleep(1000);
      break;
    default:
      break;
  }
}







